@@ -27,6 +27,10 @@ class QWidget;
 #include "HVSC_Config.h"
 #include "MixerConfig.h"
 #include "tools/IniFile.h"
+#include "SidIdConfig.h"
+#ifdef XSID_HAVE_TSID2
+#include "TSID2Config.h"
+#endif
 
 class ConfigFile : public IniFile
 {
@@ -45,11 +49,20 @@ class ConfigFile : public IniFile
     const emuConfig& getEmuConfig() const;
     const HVSC_Config& getHVSC_Config() const;
     const MixerConfig& getMixerConfig() const;
+    const SidIdConfig& getSidIdConfig() const;
+#ifdef XSID_HAVE_TSID2
+    const TSID2Config& getTSID2Config() const;
+#endif
 
     void setAudioConfig(const AudioConfig& inAudioConfig);
     void setEmuConfig(const emuConfig& inEmuConfig);
     void setHVSC_Config(const HVSC_Config& inHVSC_Config);
     void setMixerConfig(const MixerConfig& inMixerConfig);
+    void setSidIdConfig(const SidIdConfig& inSidIdConfig);
+#ifdef XSID_HAVE_TSID2
+    void setTSID2Config(const TSID2Config& inTSID2Config);
+#endif
+
 
  protected:
 
@@ -57,8 +70,11 @@ class ConfigFile : public IniFile
     emuConfig myEmuConfig;      // the emulator engine settings
     HVSC_Config myHVSC_Config;  // the HVSC/STIL settings
     MixerConfig myMixerConfig;
-    
+    SidIdConfig mySidIdConfig;  // the SidId settings
     QString fileName;
+#ifdef XSID_HAVE_TSID2
+    TSID2Config myTSID2Config;  // the TSID2 setting
+#endif
 };
 
 #endif  /* CONFIGFILE_H */
