@@ -36,6 +36,9 @@ using namespace std;
 #ifdef XSID_HAVE_TSID
 #include <tsid/tsid.h>
 #endif
+#ifdef XSID_HAVE_TSID2
+#include <tsid2/tsid2.h>
+#endif
 
 #ifdef XSID_HAVE_LIRC
 #include <unistd.h>
@@ -54,7 +57,9 @@ using namespace std;
 #include "PlaylistEditDialog.h"
 #include "PlaylistOptDialog.h"
 #include "StilDialog.h"
-#include "StilDialog.h"
+#include "SidIdDialog.h"
+#include "TSID2Dialog.h"
+
 
 #include "AudioDrivers.h"
 #include "ConfigC.h"
@@ -67,6 +72,11 @@ using namespace std;
 #include "wrapper/SongLength.h"
 #include "wrapper/EmuWrapper.h"
 #include "wrapper/SidTuneWrapper.h"
+#include "wrapper/SidTuneMod.h"
+#include "sidid/sidid.h"
+#ifdef XSID_HAVE_TSID2
+#include "TSID2Config.h"
+#endif
 
 #include "widgets/PicButton.h"
 #include "widgets/TimeLCD.h"
@@ -152,6 +162,7 @@ MainDialog::MainDialog(QWidget* parent, const char* name)
     configMenu->insertItem( "SID filter", this, SLOT(openFilterDialog()) );
 #endif
     configMenu->insertItem( "HVSC", this, SLOT(openHVSC_Dialog()) );
+    configMenu->insertItem( "SidId", this, SLOT(openSidIdDialog()) );
 
     QPopupMenu *playlistAddMenu = new QPopupMenu(this);
     Q_CHECK_PTR( playlistAddMenu );
@@ -190,6 +201,9 @@ MainDialog::MainDialog(QWidget* parent, const char* name)
 #ifndef SID_WITH_SIDPLAY2
     extraMenu->insertItem( "Mixer", this, SLOT(openMixerDialog()) );
 #endif
+#ifdef XSID_HAVE_TSID2
+    extraMenu->insertItem( "TSID2", this, SLOT(openTSID2Dialog()) );
+#endif
     
     QPopupMenu *helpMenu = new QPopupMenu(this);
     Q_CHECK_PTR( helpMenu );
@@ -213,25 +227,38 @@ MainDialog::MainDialog(QWidget* parent, const char* name)
     timeLCD->setFrameShadow( QLCDNumber::Sunken );
     timeLCD->setFrameShape( QLCDNumber::Panel );
 
+    songTimeLCD->setSmallDecimalPoint( FALSE );
+    songTimeLCD->setNumDigits( 5 );
+    songTimeLCD->setMode( QLCDNumber::DEC );
+    songTimeLCD->setSegmentStyle( QLCDNumber::Filled );
+    songTimeLCD->setFrameShadow( QLCDNumber::Sunken );
+    songTimeLCD->setFrameShape( QLCDNumber::Panel );
+
 #ifdef XSID_HAVE_TSID
     myTSID = new TSID;  // Time SID Manager
 #endif
-    
+
+    mySidId= new SidId();
+
     // The currently active file name.
     sidFileNameFull = sidFileNameHVSC = "";
 
     hotlist = new Playlist;
 
     myFileDlg = 0;  // do that when we get access to the config file
-    
+
     myAudioDlg = new AudioDialog(this);
     myEmuDlg = new EmuDialog(this);
     myFilterDlg = new FilterDialog(this);
     myMixerDlg = new MixerDialog(this);
     myHVSC_Dlg = new HVSC_Dialog(this);
+    mySidIdDlg = new SidIdDialog(this);
     myHistoryDlg = new HistoryDialog(this);
     myPlaylistDlg = new PlaylistDialog(this);
     myPlaylistEditDlg = new PlaylistEditDialog(this);
+#ifdef XSID_HAVE_TSID2
+    myTSID2Dlg=new TSID2Dialog(this);
+#endif
     
     myStilDlg = new StilDialog();
     
@@ -280,7 +307,11 @@ MainDialog::~MainDialog()
 #ifdef XSID_HAVE_TSID
     delete myTSID;
 #endif
-    
+#ifdef XSID_HAVE_TSID2
+    delete myTSID2;
+#endif
+    delete mySidId;
+
     delete next_l_pPic;
     delete play_l_pPic;
     delete stop_l_pPic;
@@ -337,13 +368,13 @@ void MainDialog::setup(ConfigFile* cfg, const QString& path, Player* p)
     connect(myPlaylistEditDlg,SIGNAL( playlistEditDelRequest(uint) ),myPlaylistDlg,SLOT( deleteListPos(uint) ));
     connect(myPlaylistEditDlg,SIGNAL( playlistEditClearRequest() ),myPlaylistDlg,SLOT( clear() ));
     connect(myPlaylistEditDlg,SIGNAL( playlistEditUpdateRequest(uint) ),myPlaylistDlg,SLOT( updateListPos(uint) ));
-    
+
     // Give player access to a few widgets.
-    player->link(this,timeLCD,myPlaylistDlg);
+    player->link(this, timeLCD, songTimeLCD, myPlaylistDlg);
     // Give playlist dialogs access to list in player.
     myPlaylistEditDlg->takeList( player->getPlaylist() );
     myPlaylistDlg->takeList( player->getPlaylist() );
-    
+
     myFilterDlg->setConfigDefault( player->getEmuEngine()->getConfigDefault() );  // *not* redundant
     // Order is important with current implementation.
     setEmuConfig( config->getEmuConfig() );
@@ -359,6 +390,11 @@ void MainDialog::setup(ConfigFile* cfg, const QString& path, Player* p)
 #endif
     setAudioConfig( config->getAudioConfig() );
     
+#ifdef XSID_HAVE_TSID2
+     myTSID2Dlg->setConfig( config->getTSID2Config() );
+#endif
+     mySidIdDlg->setConfig( config->getSidIdConfig() );
+
     // HVSC root playlist basedir hack.
     player->getPlaylist()->setMaxBasePrefix( config->getHVSC_Config().hvscRootPath.absPath() );
 
@@ -411,6 +447,24 @@ void MainDialog::setup(ConfigFile* cfg, const QString& path, Player* p)
     connect(myFileDlg,SIGNAL(fileHighlighted(const QString&)),this,SLOT(playFile(const QString&)));
     if ( config->getBool(ConfigC::keys[ConfigC::Open]) )
         openFile();
+
+#ifdef XSID_HAVE_TSID2
+     const TSID2Config& myTSID2Config=config->getTSID2Config();
+     if (myTSID2Config.custom) {
+       myTSID2 = new TSID2(myTSID2Config.engine, 
+                           myTSID2Config.number_t,
+                           myTSID2Config.number_n,
+                           myTSID2Config.absBasePath.path());  // Time SID Manager 2
+     } else {
+         myTSID2 = new TSID2(myTSID2Config.engine, 
+                             myTSID2Config.number_t,
+                             myTSID2Config.number_n);  // Time SID Manager 2
+
+       }
+#endif
+
+   // read sidid cfg file from configuration
+   mySidId->readConfig(config->getSidIdConfig().sidIdFile.filePath().ascii());
 }
 
 bool MainDialog::writeConfig()
@@ -435,6 +489,11 @@ bool MainDialog::writeConfig()
     config->setEmuConfig( player->getEmuEngine()->getConfig() );
     config->setHVSC_Config( myHVSC_Dlg->getConfig() );
     config->setMixerConfig( myMixerDlg->getConfig() );
+    
+#ifdef XSID_HAVE_TSID2
+     config->setTSID2Config( myTSID2Dlg->getConfig() );
+#endif
+    config->setSidIdConfig ( mySidIdDlg->getConfig() );
 
     // Playlist
     config->setGroup( ConfigC::groups[ConfigC::Playlist] );
@@ -445,7 +504,7 @@ bool MainDialog::writeConfig()
     // Playlist editor
     config->setGroup(ConfigC::groups[ConfigC::PlaylistEditor]);
     config->saveGeom(myPlaylistEditDlg);
-    
+
     // Selector
     config->setGroup(ConfigC::groups[ConfigC::Selector]);
     config->saveGeom(myFileDlg);
@@ -458,7 +517,7 @@ bool MainDialog::writeConfig()
     // Dialog geometries.
     config->setGroup(ConfigC::groups[ConfigC::MainWindow]);
     config->saveGeom(this);
-        
+
     bool ret = config->save();
     if (!ret)
     {
@@ -487,15 +546,15 @@ void MainDialog::setAudioConfig(const AudioConfig& inConfig)
     myAudioDlg->setConfig(inConfig);
     if ( !player->init( myAudioDlg->getConfig() ) )
         infoField->setText( player->getErrorStr() );
-        
+
     // Now retrieve the possibly changed config from the driver.
     AudioConfig goodAudioConfig = player->getAudioConfig();
     myAudioDlg->setConfig(goodAudioConfig);
-    
+
     // Set dialogs that directly depend on the emuConfig.
     myEmuDlg->setConfig( player->getEmuEngine()->getConfig() );
     myFilterDlg->setConfig( player->getEmuEngine()->getConfig() );
-    
+
     // Here update the dialog buttons according to the settings the
     // audio driver accepted.
     myAudioDlg->setButtons();
@@ -718,6 +777,13 @@ void MainDialog::showSidTuneInfo()
     tmp += tmp2;
     tmp2.sprintf("\nFormat    : %s",player->getSidTuneWrapper()->getFormatString() );
     tmp += tmp2;
+#ifdef SID_WITH_SIDPLAY2
+    tmp2.sprintf("\nPlayer    : %s",mySidId->identifyBuffer(
+      player->getSidTune()->getDataBuffer(),
+      player->getSidTune()->getDataLength()
+    ).c_str() );
+    tmp += tmp2;
+#endif
     infoField->setText(tmp);
             
     songLCD->display(player->getSidTuneWrapper()->getCurrentSong() );
@@ -733,6 +799,7 @@ void MainDialog::clearSidTuneInfo()
     tmp += "Songs     : \n";
     tmp += "Song speed: \n";
     tmp += "Format    : \n";
+    tmp += "Player    : \n";
     infoField->setText(tmp);
 }
 
@@ -746,6 +813,9 @@ void MainDialog::programExitHook()
     player->stop();
 #ifdef XSID_HAVE_TSID
     addTSID();  // add time last song has been listened to
+#endif
+#ifdef XSID_HAVE_TSID2
+    addTSID2();  // add time last song has been listened to
 #endif
     writeConfig();     // ignore return value
     myHistoryDlg->save();
@@ -1139,18 +1209,32 @@ void MainDialog::about()
 {
     QString verCat = "SIDPLAY/X11   Version ";
     verCat += VERSION;
+    verCat += "\nby Simon White & Stefano Tognon\n\n";
+
+    verCat += "from SIDPLAY/X11   Version ";
+    verCat += "1.6.5.2";
     verCat += "\nby Michael Schwendt\n\n";
     
     verCat += player->getEmuEngine()->getVersionString();
     verCat += "\n\n";
     
     verCat += mySTIL.getVersion();
+
+#ifdef SID_WITH_SIDPLAY2
+    verCat += "\n";
+    verCat += mySidId->getVersion();
+    verCat += "\n";
+#endif
     
 #ifdef XSID_HAVE_TSID
     verCat += "\n";
     verCat += myTSID->getTSIDVersion();
 #endif
-    
+#ifdef XSID_HAVE_TSID2
+    verCat += "\n";
+    verCat += myTSID2->getTSID2Version();
+#endif
+
     verCat += "\nListening mileage: ";
     long int mileage = usage+player->getEmuEngine()->getSecondsTotal();
     int days = mileage/(3600L*24);
@@ -1208,11 +1292,26 @@ void MainDialog::openHVSC_Dialog()
     myHVSC_Dlg->show();
 }
 
+void MainDialog::openSidIdDialog()
+{
+    mySidIdDlg->setSidId(mySidId);
+    mySidIdDlg->show();
+}
+
 void MainDialog::openHistoryDialog()
 {
     myHistoryDlg->show();
 }
 
+void MainDialog::openTSID2Dialog()
+{
+#ifdef XSID_HAVE_TSID2
+    myTSID2Dlg->update(myTSID2->getTblist());
+    myTSID2Dlg->show();
+#endif
+}
+
+
 // --------------------------------------------------------------------------
 
 /* slot */
@@ -1242,6 +1341,9 @@ void MainDialog::newSubSong(int newSong)
         player->pause();
 #ifdef XSID_HAVE_TSID
         addTSID();  // add time last song has been listened to
+#endif
+#ifdef XSID_HAVE_TSID2
+        addTSID2();  // add time last song has been listened to
 #endif
         resetTimeDisplay();
         player->initSong(newSong);
@@ -1292,6 +1394,9 @@ void MainDialog::prevSongPressed()
         player->pause();
 #ifdef XSID_HAVE_TSID
         addTSID();  // add time last song has been listened to
+#endif
+#ifdef XSID_HAVE_TSID2
+        addTSID2();  // add time last song has been listened to
 #endif
         resetTimeDisplay();
         player->initPrevSong();
@@ -1328,6 +1433,9 @@ void MainDialog::nextSongPressed()
         player->pause();
 #ifdef XSID_HAVE_TSID
         addTSID();  // add time last song has been listened to
+#endif
+#ifdef XSID_HAVE_TSID2
+        addTSID2();  // add time last song has been listened to
 #endif
         resetTimeDisplay();
         player->initNextSong();
@@ -1360,6 +1468,9 @@ void MainDialog::stopSongPressed()
 {
     if ( player->isReady() && player->isPlaying() )
     {
+#ifdef XSID_HAVE_TSID2 
+        addTSID2();   // for true realtime list generation
+#endif
         stopBtn->setPixmap(*stop_l_pPic);
         playBtn->setPixmap(*play_lPic);
         ffBtn->setPixmap(*ffPic);
@@ -1656,6 +1767,9 @@ void MainDialog::playFileMain(const QString& fileName, int song,
     player->stop();
 #ifdef XSID_HAVE_TSID
     addTSID();  // add time last song has been listened to
+#endif
+#ifdef XSID_HAVE_TSID2
+    addTSID2();  // add time last song has been listened to
 #endif
     resetTimeDisplay();
 
@@ -1763,14 +1877,31 @@ void MainDialog::addTSID()
     static int prevTotalSecs = 0;
     int thisSecs = player->getEmuEngine()->getSecondsTotal() - prevTotalSecs;
     bool ret = myTSID->addTime( sidFileNameHVSC, thisSecs, player->getSidTuneWrapper()->getCurrentSong() );
-#ifdef XSID_WB_DEBUG 
+#ifdef XSID_WB_DEBUG
     cout << "TSID: " << sidFileNameHVSC << " / " << thisSecs << " / " << '#' << player->getSidTuneWrapper()->getCurrentSong() << " / return = " << ret << endl;
 #endif
     prevTotalSecs = player->getEmuEngine()->getSecondsTotal();
-    
+
 }
 #endif
 
+#ifdef XSID_HAVE_TSID2
+void MainDialog::addTSID2()
+{
+    static int prevTotalSecs = 0;
+    int thisSecs = player->getEmuEngine()->getSecondsTotal() - prevTotalSecs;
+    bool ret = myTSID2->addTime( player->getSidTuneWrapper()->getMD5_Digest(),
+                                 sidFileNameFull, thisSecs,
+                                 player->getSidTuneWrapper()->getCurrentSong(),
+                                 querySongLengthDB(player->getSidTuneWrapper(), player->getSidTuneWrapper()->getCurrentSong()));
+
+#ifdef XSID_WB_DEBUG
+    cout << "TSID2: " << sidFileNameFull << " / " << thisSecs << " / " << '#' << player->getSidTuneWrapper()->getCurrentSong() << " / return = " << ret << endl;
+#endif
+    prevTotalSecs = player->getEmuEngine()->getSecondsTotal();
+
+}
+#endif
 
 // --------------------------------------------------------------------- LIRC
 
@@ -1873,3 +2004,4 @@ void MainDialog::lircEvent(int fh)
 
 #endif  // XSID_HAVE_LIRC
 }
+
