@@ -37,6 +37,7 @@ using namespace std;
 #include "TimeLCD.h"
 #include "wrapper/EmuWrapper.h"
 #include "wrapper/SidTuneWrapper.h"
+#include "wrapper/SongLength.h"
 
 Player::Player()
 {
@@ -98,11 +99,12 @@ void Player::writeConfig(ConfigFile* config)
 {
 }
 
-void Player::link(MainDialog* mainDlg, TimeLCD* timeLCD, 
+void Player::link(MainDialog* mainDlg, TimeLCD* timeLCD, TimeLCD* songTimeLCD,
                   PlaylistDialog* playlistDlg)
 {
     myMainDlg = mainDlg;  // not used
     myTimeLCD = timeLCD;
+    mySongTimeLCD = songTimeLCD;
     myPlaylistDlg = playlistDlg;
 }
 
@@ -276,6 +278,16 @@ void Player::initSong(int song)
 {
     pEmuEngine->initSong(pSidTuneWrapper->getSidTune(),song);
     pSidTuneWrapper->updateInfo();
+
+    // update the song length tiem LCD 
+    int secs=0;
+
+    SongLengthDBitem sli;
+    if (SongLength::getItem( pSidTuneWrapper, song, sli )) {
+      secs=sli.playtime;
+    }
+    mySongTimeLCD->resetDisplay();
+    mySongTimeLCD->updateDisplay(secs);
 }
 
 void Player::initNextSong()
