@@ -63,7 +63,7 @@ class Player : public QObject
     void readConfig(ConfigFile*);
     void writeConfig(ConfigFile*);
     
-    void link(MainDialog*, TimeLCD*, PlaylistDialog*);
+    void link(MainDialog*, TimeLCD*, TimeLCD*, PlaylistDialog*);
     bool init(const AudioConfig&);
     const QString& getErrorStr();
          
@@ -166,6 +166,7 @@ class Player : public QObject
 
     MainDialog* myMainDlg;
     TimeLCD* myTimeLCD;
+    TimeLCD* mySongTimeLCD;
     PlaylistDialog* myPlaylistDlg;
     
     bool audioBuffersReady;
