@@ -22,11 +22,11 @@
 #include "StilDialog.h"
 #include "GlobalIcon.h"
 
-StilDialog::StilDialog(QWidget* mainWin, QWidget* parent, const char* name)
+StilDialog::StilDialog(QWidget* parent, const char* name)
 : QDialog(parent,name)
 {
-    main = mainWin;
-    
+    haveContent = false;
+        
     setCaption( "STIL view" );
     setBackgroundMode(PaletteBackground);
     setIcon(*mainIcon);
@@ -148,7 +148,8 @@ void StilDialog::setLabels(const char* global,
         bugLabel->clear();
     }
 
-    if ( contentsWidth!=0 && contentsHeight!=0 )
+    haveContent = ( contentsWidth!=0 && contentsHeight!=0 );
+    if ( haveContent )
     {
         // Right margin (scrollbar safety).
         contentsWidth += 20;
@@ -170,8 +171,7 @@ void StilDialog::setLabels(const char* global,
                 dlgHeight = maxHeight;
             resize(contentsWidth,dlgHeight);
             scrollView->resize(width(),height());
-            if ( main->isVisible() )
-                show();
+            show();
         }
     }
     else  // no contents
