@@ -3,28 +3,31 @@
 //
 // AUTHOR: LaLa
 // Email : LaLa@C64.org
-// (C) 1998-2000 by LaLa
+// Copyright (C) 1998, 2002 by LaLa
 //
 
 #ifndef _STIL
 #define _STIL
 
-#include <iostream.h>
-#include <string.h>
-#include <fstream.h>
-#include <stdio.h>      // For sprintf(), scanf() and NULL
+#include <iostream>
+#include <iomanip>
+#include <cstring>
+#include <fstream>
+#include <cstdio>      // For snprintf() and NULL
+using namespace std;
 #include "stil.h"
 
-#define STILopenFlags (ios::in | ios::binary | ios::nocreate)
+#define STILopenFlags (ios::in | ios::binary)
 
-const float VERSION_NO = 2.11;
+const float VERSION_NO = 2.17;
 
 #define CERR_STIL_DEBUG if (STIL_DEBUG) cerr << "Line #" << __LINE__ << " STIL::"
 
 // CONSTRUCTOR
 STIL::STIL()
 {
-    sprintf(versionString, "STILView v%4.2f, (C) 1998-2000 by LaLa (LaLa@C64.org)\n", VERSION_NO);
+    snprintf(versionString, 2*STIL_MAX_LINE_SIZE-1, "STILView v%4.2f, (C) 1998, 2002 by LaLa (LaLa@C64.org)\n", VERSION_NO);
+    versionString[2*STIL_MAX_LINE_SIZE-1] = '\0';
 
     STILVersion = 0.0;
     baseDir = NULL;
@@ -89,6 +92,8 @@ STIL::setBaseDir(const char *pathToHVSC)
     char *tempName;     // Points to full pathname to STIL.txt.
     char *tempBaseDir;
     size_t tempBaseDirLength;
+    size_t pathToHVSCLength = strlen(pathToHVSC);
+    size_t tempNameLength;
 
     // Temporary version number/copyright string.
     char tempVersionString[2*STIL_MAX_LINE_SIZE];
@@ -111,8 +116,9 @@ STIL::setBaseDir(const char *pathToHVSC)
         return false;
     }
 
-    tempBaseDir = new char [strlen(pathToHVSC)+1];
-    strcpy(tempBaseDir, pathToHVSC);
+    tempBaseDir = new char [pathToHVSCLength+1];
+    strncpy(tempBaseDir, pathToHVSC, pathToHVSCLength);
+    tempBaseDir[pathToHVSCLength] = '\0';
 
     // Chop the trailing slash
     temp = tempBaseDir+strlen(tempBaseDir)-1;
@@ -124,9 +130,11 @@ STIL::setBaseDir(const char *pathToHVSC)
     // Attempt to open STIL
 
     // Create the full path+filename
-    tempName = new char [tempBaseDirLength+strlen(PATH_TO_STIL)+1];
-    strcpy(tempName, tempBaseDir);
-    strcat(tempName, PATH_TO_STIL);
+    tempNameLength = tempBaseDirLength+strlen(PATH_TO_STIL);
+    tempName = new char [tempNameLength+1];
+    strncpy(tempName, tempBaseDir, tempNameLength);
+    strncat(tempName, PATH_TO_STIL, tempNameLength-tempBaseDirLength);
+    tempName[tempNameLength] = '\0';
     convertSlashes(tempName);
 
     stilFile.clear();
@@ -145,9 +153,11 @@ STIL::setBaseDir(const char *pathToHVSC)
 
     // Create the full path+filename
     delete[] tempName;
-    tempName = new char [tempBaseDirLength+strlen(PATH_TO_BUGLIST)+1];
-    strcpy(tempName, tempBaseDir);
-    strcat(tempName, PATH_TO_BUGLIST);
+    tempNameLength = tempBaseDirLength+strlen(PATH_TO_BUGLIST);
+    tempName = new char [tempNameLength+1];
+    strncpy(tempName, tempBaseDir, tempNameLength);
+    strncat(tempName, PATH_TO_BUGLIST, tempNameLength-tempBaseDirLength);
+    tempName[tempNameLength] = '\0';
     convertSlashes(tempName);
 
     bugFile.clear();
@@ -177,8 +187,10 @@ STIL::setBaseDir(const char *pathToHVSC)
     }
 
     // Save away the current string so we can restore it if needed.
-    strcpy(tempVersionString, versionString);
-    sprintf(versionString, "STILView v%4.2f, (C) 1998-2000 by LaLa (LaLa@C64.org)\n", VERSION_NO);
+    strncpy(tempVersionString, versionString, 2*STIL_MAX_LINE_SIZE-1);
+    tempVersionString[2*STIL_MAX_LINE_SIZE-1] = '\0';
+    snprintf(versionString, 2*STIL_MAX_LINE_SIZE-1, "STILView v%4.2f, (C) 1998, 2002 by LaLa (LaLa@C64.org)\n", VERSION_NO);
+    versionString[2*STIL_MAX_LINE_SIZE-1] = '\0';
 
     // This is necessary so the version number gets scanned in from the new
     // file, too.
@@ -199,23 +211,20 @@ STIL::setBaseDir(const char *pathToHVSC)
         deleteDirList(tempStilDirs);
         deleteDirList(tempBugDirs);
         STILVersion = tempSTILVersion;
-        strcpy(versionString, tempVersionString);
+        strncpy(versionString, tempVersionString, 2*STIL_MAX_LINE_SIZE-1);
+        versionString[2*STIL_MAX_LINE_SIZE-1] = '\0';
         return false;
     }
 
     if (bugFile.good()) {
         if (getDirs(bugFile, tempBugDirs, false) != true) {
-            stilFile.close();
-            bugFile.close();
-            CERR_STIL_DEBUG << "getDirs() failed for bugFile" << endl;
-            lastError = NO_BUG_DIRS;
 
-            // Clean up and restore things.
-            deleteDirList(tempStilDirs);
-            deleteDirList(tempBugDirs);
-            STILVersion = tempSTILVersion;
-            strcpy(versionString, tempVersionString);
-            return false;
+            // This is not a critical error - it is possible that the
+            // BUGlist.txt file has no entries in it at all (in fact, that's
+            // good!).
+
+            CERR_STIL_DEBUG << "getDirs() failed for bugFile" << endl;
+            lastError = BUG_OPEN;
         }
     }
 
@@ -232,8 +241,9 @@ STIL::setBaseDir(const char *pathToHVSC)
     }
 
     // Copy.
-    baseDir = new char [strlen(tempBaseDir)+1];
-    strcpy(baseDir, tempBaseDir);
+    baseDir = new char [tempBaseDirLength+1];
+    strncpy(baseDir, tempBaseDir, tempBaseDirLength);
+    baseDir[tempBaseDirLength] = '\0';
     baseDirLength = tempBaseDirLength;
 
     // First, delete whatever may have been there previously.
@@ -248,6 +258,11 @@ STIL::setBaseDir(const char *pathToHVSC)
     copyDirList(stilDirs, tempStilDirs);
     copyDirList(bugDirs, tempBugDirs);
 
+    // Clear the buffers (caches).
+    memset((void *)entrybuf, 0, sizeof(entrybuf));
+    memset((void *)globalbuf, 0, sizeof(globalbuf));
+    memset((void *)bugbuf, 0, sizeof(bugbuf));
+
     // Cleanup.
     deleteDirList(tempStilDirs);
     deleteDirList(tempBugDirs);
@@ -263,21 +278,30 @@ STIL::getAbsEntry(const char *absPathToEntry, int tuneNo, STILField field)
 {
     char *tempDir;
     char *returnPtr;
+    size_t tempDirLength;
 
     lastError = NO_STIL_ERROR;
 
     CERR_STIL_DEBUG << "getAbsEntry() called, absPathToEntry=" << absPathToEntry << endl;
 
+    if (baseDir == NULL) {
+        CERR_STIL_DEBUG << "HVSC baseDir is not yet set!" << endl;
+        lastError = STIL_OPEN;
+        return NULL;
+    }
+
     // Determine if the baseDir is in the given pathname.
 
-    if (MYSTRNCMP(absPathToEntry, baseDir, baseDirLength) != 0) {
+    if (MYSTRNICMP(absPathToEntry, baseDir, baseDirLength) != 0) {
         CERR_STIL_DEBUG << "getAbsEntry() failed: baseDir=" << baseDir << ", absPath=" << absPathToEntry << endl;
         lastError = WRONG_DIR;
         return NULL;
     }
 
-    tempDir = new char [strlen(absPathToEntry)-baseDirLength+1];
-    strcpy(tempDir, absPathToEntry+baseDirLength);
+    tempDirLength = strlen(absPathToEntry)-baseDirLength;
+    tempDir = new char [tempDirLength+1];
+    strncpy(tempDir, absPathToEntry+baseDirLength, tempDirLength);
+    tempDir[tempDirLength] = '\0';
     convertToSlashes(tempDir);
 
     returnPtr = getEntry(tempDir, tuneNo, field);
@@ -289,15 +313,16 @@ char *
 STIL::getEntry(const char *relPathToEntry, int tuneNo, STILField field)
 {
     char *tempName;
+    size_t tempNameLength;
 
     lastError = NO_STIL_ERROR;
 
-    if (STILVersion < 2.59) {
+    CERR_STIL_DEBUG << "getEntry() called, relPath=" << relPathToEntry << ", rest=" << tuneNo << "," << field << endl;
 
-        // Older version of STIL is detected.
-
-        tuneNo = 0;
-        field = all;
+    if (baseDir == NULL) {
+        CERR_STIL_DEBUG << "HVSC baseDir is not yet set!" << endl;
+        lastError = STIL_OPEN;
+        return NULL;
     }
 
     // Fail if a section-global comment was asked for.
@@ -308,11 +333,17 @@ STIL::getEntry(const char *relPathToEntry, int tuneNo, STILField field)
         return NULL;
     }
 
-    CERR_STIL_DEBUG << "getEntry() called, relPath=" << relPathToEntry << ", rest=" << tuneNo << "," << field << endl;
+    if (STILVersion < 2.59) {
+
+        // Older version of STIL is detected.
+
+        tuneNo = 0;
+        field = all;
+    }
 
     // Find out whether we have this entry in the buffer.
 
-    if ((MYSTRNCMP(entrybuf, relPathToEntry, strlen(relPathToEntry)) != 0) ||
+    if ((MYSTRNICMP(entrybuf, relPathToEntry, strlen(relPathToEntry)) != 0) ||
         ((( (size_t) (strchr(entrybuf, '\n')-entrybuf)) != strlen(relPathToEntry))
             && (STILVersion > 2.59))) {
 
@@ -322,9 +353,11 @@ STIL::getEntry(const char *relPathToEntry, int tuneNo, STILField field)
         CERR_STIL_DEBUG << "getEntry(): entry not in buffer" << endl;
 
         // Create the full path+filename
-        tempName = new char [baseDirLength+strlen(PATH_TO_STIL)+1];
-        strcpy(tempName, baseDir);
-        strcat(tempName, PATH_TO_STIL);
+        tempNameLength = baseDirLength+strlen(PATH_TO_STIL);
+        tempName = new char [tempNameLength+1];
+        strncpy(tempName, baseDir, tempNameLength);
+        strncat(tempName, PATH_TO_STIL, tempNameLength-baseDirLength);
+        tempName[tempNameLength] = '\0';
         convertSlashes(tempName);
 
         stilFile.clear();
@@ -342,8 +375,9 @@ STIL::getEntry(const char *relPathToEntry, int tuneNo, STILField field)
 
         if (positionToEntry(relPathToEntry, stilFile, stilDirs) == false) {
             // Copy the entry's name to the buffer.
-            strcpy(entrybuf, relPathToEntry);
-            addNewline(entrybuf);
+            strncpy(entrybuf, relPathToEntry, STIL_MAX_ENTRY_SIZE-1);
+            strncat(entrybuf, "\n", 2);
+            entrybuf[STIL_MAX_ENTRY_SIZE-1] = '\0';
             CERR_STIL_DEBUG << "getEntry() posToEntry() failed" << endl;
             lastError = NOT_IN_STIL;
         }
@@ -372,21 +406,30 @@ STIL::getAbsBug(const char *absPathToEntry, int tuneNo)
 {
     char *tempDir;
     char *returnPtr;
+    size_t tempDirLength;
 
     lastError = NO_STIL_ERROR;
 
     CERR_STIL_DEBUG << "getAbsBug() called, absPathToEntry=" << absPathToEntry << endl;
 
+    if (baseDir == NULL) {
+        CERR_STIL_DEBUG << "HVSC baseDir is not yet set!" << endl;
+        lastError = BUG_OPEN;
+        return NULL;
+    }
+
     // Determine if the baseDir is in the given pathname.
 
-    if (MYSTRNCMP(absPathToEntry, baseDir, baseDirLength) != 0) {
+    if (MYSTRNICMP(absPathToEntry, baseDir, baseDirLength) != 0) {
         CERR_STIL_DEBUG << "getAbsBug() failed: baseDir=" << baseDir << ", absPath=" << absPathToEntry << endl;
         lastError = WRONG_DIR;
         return NULL;
     }
 
-    tempDir = new char [strlen(absPathToEntry)-baseDirLength+1];
-    strcpy(tempDir, absPathToEntry+baseDirLength);
+    tempDirLength = strlen(absPathToEntry)-baseDirLength;
+    tempDir = new char [tempDirLength+1];
+    strncpy(tempDir, absPathToEntry+baseDirLength, tempDirLength);
+    tempDir[tempDirLength] = '\0';
     convertToSlashes(tempDir);
 
     returnPtr = getBug(tempDir, tuneNo);
@@ -398,20 +441,29 @@ char *
 STIL::getBug(const char *relPathToEntry, int tuneNo)
 {
     char *tempName;
+    size_t tempNameLength;
 
     lastError = NO_STIL_ERROR;
 
+    CERR_STIL_DEBUG << "getBug() called, relPath=" << relPathToEntry << ", rest=" << tuneNo << endl;
+
+    if (baseDir == NULL) {
+        CERR_STIL_DEBUG << "HVSC baseDir is not yet set!" << endl;
+        lastError = BUG_OPEN;
+        return NULL;
+    }
+
     // Older version of STIL is detected.
 
     if (STILVersion < 2.59) {
         tuneNo = 0;
     }
 
-    CERR_STIL_DEBUG << "getBug() called, relPath=" << relPathToEntry << ", rest=" << tuneNo << endl;
-
     // Find out whether we have this bug entry in the buffer.
+    // If the baseDir was changed, we'll have to read it in again,
+    // even if it might be in the buffer already.
 
-    if ((MYSTRNCMP(bugbuf, relPathToEntry, strlen(relPathToEntry)) != 0) ||
+    if ((MYSTRNICMP(bugbuf, relPathToEntry, strlen(relPathToEntry)) != 0) ||
         ((( (size_t) (strchr(bugbuf, '\n')-bugbuf)) != strlen(relPathToEntry)) &&
             (STILVersion > 2.59))) {
 
@@ -421,9 +473,11 @@ STIL::getBug(const char *relPathToEntry, int tuneNo)
         CERR_STIL_DEBUG << "getBug(): entry not in buffer" << endl;
 
         // Create the full path+filename
-        tempName = new char [baseDirLength+strlen(PATH_TO_BUGLIST)+1];
-        strcpy(tempName, baseDir);
-        strcat(tempName, PATH_TO_BUGLIST);
+        tempNameLength = baseDirLength+strlen(PATH_TO_BUGLIST);
+        tempName = new char [tempNameLength+1];
+        strncpy(tempName, baseDir, tempNameLength);
+        strncat(tempName, PATH_TO_BUGLIST, tempNameLength-baseDirLength);
+        tempName[tempNameLength] = '\0';
         convertSlashes(tempName);
 
         bugFile.clear();
@@ -441,8 +495,9 @@ STIL::getBug(const char *relPathToEntry, int tuneNo)
 
         if (positionToEntry(relPathToEntry, bugFile, bugDirs) == false) {
             // Copy the entry's name to the buffer.
-            strcpy(bugbuf, relPathToEntry);
-            addNewline(bugbuf);
+            strncpy(bugbuf, relPathToEntry, STIL_MAX_ENTRY_SIZE-1);
+            strncat(bugbuf, "\n", 2);
+            bugbuf[STIL_MAX_ENTRY_SIZE-1] = '\0';
             CERR_STIL_DEBUG << "getBug() posToEntry() failed" << endl;
             lastError = NOT_IN_BUG;
         }
@@ -471,21 +526,30 @@ STIL::getAbsGlobalComment(const char *absPathToEntry)
 {
     char *tempDir;
     char *returnPtr;
+    size_t tempDirLength;
 
     lastError = NO_STIL_ERROR;
 
     CERR_STIL_DEBUG << "getAbsGC() called, absPathToEntry=" << absPathToEntry << endl;
 
+    if (baseDir == NULL) {
+        CERR_STIL_DEBUG << "HVSC baseDir is not yet set!" << endl;
+        lastError = STIL_OPEN;
+        return NULL;
+    }
+
     // Determine if the baseDir is in the given pathname.
 
-    if (MYSTRNCMP(absPathToEntry, baseDir, baseDirLength) != 0) {
+    if (MYSTRNICMP(absPathToEntry, baseDir, baseDirLength) != 0) {
         CERR_STIL_DEBUG << "getAbsGC() failed: baseDir=" << baseDir << ", absPath=" << absPathToEntry << endl;
         lastError = WRONG_DIR;
         return NULL;
     }
 
-    tempDir = new char [strlen(absPathToEntry)-baseDirLength+1];
-    strcpy(tempDir, absPathToEntry+baseDirLength);
+    tempDirLength = strlen(absPathToEntry)-baseDirLength;
+    tempDir = new char [tempDirLength+1];
+    strncpy(tempDir, absPathToEntry+baseDirLength, tempDirLength);
+    tempDir[tempDirLength] = '\0';
     convertToSlashes(tempDir);
 
     returnPtr = getGlobalComment(tempDir);
@@ -499,6 +563,7 @@ STIL::getGlobalComment(const char *relPathToEntry)
     char *dir;
     char *tempName;
     size_t pathLen;
+    size_t tempNameLength;
     char *temp;
     char *lastSlash;
 
@@ -506,6 +571,12 @@ STIL::getGlobalComment(const char *relPathToEntry)
 
     CERR_STIL_DEBUG << "getGC() called, relPath=" << relPathToEntry << endl;
 
+    if (baseDir == NULL) {
+        CERR_STIL_DEBUG << "HVSC baseDir is not yet set!" << endl;
+        lastError = STIL_OPEN;
+        return NULL;
+    }
+
     // Save the dirpath.
 
     lastSlash = (char *)strrchr(relPathToEntry, '/');
@@ -521,8 +592,10 @@ STIL::getGlobalComment(const char *relPathToEntry)
     *(dir+pathLen) = '\0';
 
     // Find out whether we have this global comment in the buffer.
+    // If the baseDir was changed, we'll have to read it in again,
+    // even if it might be in the buffer already.
 
-    if ((MYSTRNCMP(globalbuf, dir, pathLen) != 0) ||
+    if ((MYSTRNICMP(globalbuf, dir, pathLen) != 0) ||
         ((( (size_t) (strchr(globalbuf, '\n')-globalbuf)) != pathLen) &&
             (STILVersion > 2.59))) {
 
@@ -532,9 +605,11 @@ STIL::getGlobalComment(const char *relPathToEntry)
         CERR_STIL_DEBUG << "getGC(): entry not in buffer" << endl;
 
         // Create the full path+filename
-        tempName = new char [baseDirLength+strlen(PATH_TO_STIL)+1];
-        strcpy(tempName, baseDir);
-        strcat(tempName, PATH_TO_STIL);
+        tempNameLength = baseDirLength+strlen(PATH_TO_STIL);
+        tempName = new char [tempNameLength+1];
+        strncpy(tempName, baseDir, tempNameLength);
+        strncat(tempName, PATH_TO_STIL, tempNameLength-baseDirLength);
+        tempName[tempNameLength] = '\0';
         convertSlashes(tempName);
 
         stilFile.clear();
@@ -551,8 +626,9 @@ STIL::getGlobalComment(const char *relPathToEntry)
 
         if (positionToEntry(dir, stilFile, stilDirs) == false) {
             // Copy the dirname to the buffer.
-            strcpy(globalbuf, dir);
-            addNewline(globalbuf);
+            strncpy(globalbuf, dir, STIL_MAX_ENTRY_SIZE-1);
+            strncat(globalbuf, "\n", 2);
+            globalbuf[STIL_MAX_ENTRY_SIZE-1] = '\0';
             CERR_STIL_DEBUG << "getGC() posToEntry() failed" << endl;
             lastError = NOT_IN_STIL;
         }
@@ -674,11 +750,13 @@ STIL::getDirs(ifstream& inFile, dirList *dirs, bool isSTILFile)
             if (strncmp(line, "#  STIL v", 9) == 0) {
 
                 // Get the version number
-                sscanf(line+9, "%f", &STILVersion);
+                STILVersion = atof(line+9);
 
                 // Put it into the string, too.
-                sprintf(line, "SID Tune Information List (STIL) v%4.2f\n", STILVersion);
-                strcat(versionString, line);
+                snprintf(line, STIL_MAX_LINE_SIZE-1, "SID Tune Information List (STIL) v%4.2f\n", STILVersion);
+                line[STIL_MAX_LINE_SIZE-1] = '\0';
+                strncat(versionString, line, 2*STIL_MAX_LINE_SIZE-1);
+                versionString[2*STIL_MAX_LINE_SIZE-1] = '\0';
 
                 CERR_STIL_DEBUG << "getDirs() STILVersion=" << STILVersion << endl;
 
@@ -688,7 +766,7 @@ STIL::getDirs(ifstream& inFile, dirList *dirs, bool isSTILFile)
 
         // Search for the start of a dir separator first.
 
-        if (isSTILFile && !newDir && (MYSTRNCMP(line, "### ", 4) == 0)) {
+        if (isSTILFile && !newDir && (MYSTRNICMP(line, "### ", 4) == 0)) {
             newDir = true;
             continue;
         }
@@ -705,7 +783,7 @@ STIL::getDirs(ifstream& inFile, dirList *dirs, bool isSTILFile)
                 if (i==0) {
                     newDir = true;
                 }
-                else if (MYSTRNCMP(prevDir->dirName, line, j) != 0) {
+                else if (MYSTRNICMP(prevDir->dirName, line, j) != 0) {
                     newDir = true;
                 }
                 else {
@@ -720,7 +798,7 @@ STIL::getDirs(ifstream& inFile, dirList *dirs, bool isSTILFile)
                 strncpy(dirs->dirName, line, j);
                 *((dirs->dirName)+j) = '\0';
 
-                dirs->position = inFile.tellg()-strlen(line)-1;
+                dirs->position = inFile.tellg()-(streampos)strlen(line)-1L;
 
                 CERR_STIL_DEBUG << "getDirs() i=" << i << ", dirName=" << dirs->dirName << ", pos=" << dirs->position <<  endl;
 
@@ -744,7 +822,9 @@ STIL::getDirs(ifstream& inFile, dirList *dirs, bool isSTILFile)
     }
 
     if (i == 0) {
-        // No entries found - something is wrong
+        // No entries found - something is wrong.
+        // NOTE: It's perfectly valid to have a BUGlist.txt file with no
+        // entries in it!
         CERR_STIL_DEBUG << "getDirs() no dirs found" << endl;
         return false;
     }
@@ -791,7 +871,7 @@ STIL::positionToEntry(const char *entryStr, ifstream& inFile, dirList *dirs)
     // Find it in the table.
 
     while (dirs->dirName) {
-        if ((MYSTRNCMP(dirs->dirName, entryStr, pathLen) == 0) &&
+        if ((MYSTRNICMP(dirs->dirName, entryStr, pathLen) == 0) &&
             (strlen(dirs->dirName) == pathLen)) {
             CERR_STIL_DEBUG << "pos2Entry() found dir, dirName=" << dirs->dirName << endl;
             foundIt = true;
@@ -822,7 +902,7 @@ STIL::positionToEntry(const char *entryStr, ifstream& inFile, dirList *dirs)
 
         if (*line == '/') {
 
-            if (MYSTRNCMP(dirs->dirName, line, pathLen) != 0) {
+            if (MYSTRNICMP(dirs->dirName, line, pathLen) != 0) {
                 // We are outside the section - get out of the loop,
                 // which will fail the search.
                 break;
@@ -832,13 +912,13 @@ STIL::positionToEntry(const char *entryStr, ifstream& inFile, dirList *dirs)
             // a specific entry.
 
             if (globComm || (STILVersion > 2.59)) {
-                temp = MYSTRCMP(line, entryStr);
+                temp = MYSTRICMP(line, entryStr);
             }
             else {
                 // To be compatible with older versions of STIL, which may have
                 // the tune designation on the first line of a STIL entry
                 // together with the pathname.
-                temp = MYSTRNCMP(line, entryStr, entryStrLen);
+                temp = MYSTRNICMP(line, entryStr, entryStrLen);
             }
 
             CERR_STIL_DEBUG << "pos2Entry() line=" << line << endl;
@@ -853,7 +933,7 @@ STIL::positionToEntry(const char *entryStr, ifstream& inFile, dirList *dirs)
 
     if (foundIt) {
         // Reposition the file pointer back to the start of the entry.
-        inFile.seekg(inFile.tellg()-strlen(line)-1);
+        inFile.seekg(inFile.tellg()-(streampos)strlen(line)-1L);
         CERR_STIL_DEBUG << "pos2Entry() entry found" << endl;
         return true;
     }
@@ -872,7 +952,7 @@ STIL::readEntry(ifstream& inFile, char *buffer)
         getStilLine(inFile, line);
         strcat(buffer, line);
         if (*line != '\0') {
-            addNewline(buffer);
+            strncat(buffer, "\n", 2);
         }
     } while (*line != '\0');
 }
@@ -880,7 +960,7 @@ STIL::readEntry(ifstream& inFile, char *buffer)
 bool
 STIL::getField(char *result, char *buffer, int tuneNo, STILField field)
 {
-    char *start, *firstTuneNo, *temp;
+    char *start, *firstTuneNo, *temp, *temp2 = NULL;
 
     CERR_STIL_DEBUG << "getField() called, buffer=" << buffer << ", rest=" << tuneNo << "," << field << endl;
 
@@ -914,24 +994,58 @@ STIL::getField(char *result, char *buffer, int tuneNo, STILField field)
         // SINGLE TUNE ENTRY //
         //-------------------//
 
-        // Is the only thing in this STIL entry the COMMENT?
+        // Is the first thing in this STIL entry the COMMENT?
+
         temp = strstr(start, _COMMENT_STR);
 
+        // Search for other potential fields beyond the COMMENT.
+        if (temp == start) {
+            temp2 = strstr(start, _NAME_STR);
+            if (temp2 == NULL) {
+                temp2 = strstr(start, _AUTHOR_STR);
+                if (temp2 == NULL) {
+                    temp2 = strstr(start, _TITLE_STR);
+                    if (temp2 == NULL) {
+                        temp2 = strstr(start, _ARTIST_STR);
+                    }
+                }
+            }
+        }
+
         if (temp == start) {
 
-            // Yes. So it's assumed to be a tune-global comment.
+            // Yes. So it's assumed to be a file-global comment.
 
             CERR_STIL_DEBUG << "getField() single-tune entry, COMMENT only" << endl;
 
-            if ((tuneNo == 0) && ((field == comment) || (field == all))) {
+            if ((tuneNo == 0) && ((field == all) || ((field == comment) && (temp2 == NULL))) ) {
 
                 // Simply copy the stuff in.
 
-                strcpy(result, start);
+                strncpy(result, start, STIL_MAX_ENTRY_SIZE-1);
+                result[STIL_MAX_ENTRY_SIZE-1] = '\0';
                 CERR_STIL_DEBUG << "getField() copied to resultbuf" << endl;
                 return true;
             }
 
+            else if ((tuneNo == 0) && (field == comment)) {
+
+                // Copy just the comment.
+
+                strncpy(result, start, temp2-start);
+                *(result+(temp2-start)) = '\0';
+                CERR_STIL_DEBUG << "getField() copied to just the COMMENT to resultbuf" << endl;
+                return true;
+            }
+
+            else if ((tuneNo == 1) && (temp2 != NULL)) {
+
+               // A specific field was asked for.
+
+                CERR_STIL_DEBUG << "getField() copying COMMENT to resultbuf" << endl;
+                return getOneField(result, temp2, temp2+strlen(temp2), field);
+            }
+
             else {
 
                 // Anything else is invalid as of v2.00.
@@ -950,7 +1064,8 @@ STIL::getField(char *result, char *buffer, int tuneNo, STILField field)
 
                 // The complete entry was asked for. Simply copy the stuff in.
 
-                strcpy(result, start);
+                strncpy(result, start, STIL_MAX_ENTRY_SIZE-1);
+                result[STIL_MAX_ENTRY_SIZE-1] = '\0';
                 CERR_STIL_DEBUG << "getField() copied to resultbuf" << endl;
                 return true;
             }
@@ -990,21 +1105,22 @@ STIL::getField(char *result, char *buffer, int tuneNo, STILField field)
 
                     // Yes. Simply copy the stuff in.
 
-                    strcpy(result, start);
+                    strncpy(result, start, STIL_MAX_ENTRY_SIZE-1);
+                    result[STIL_MAX_ENTRY_SIZE-1] = '\0';
                     CERR_STIL_DEBUG << "getField() copied all to resultbuf" << endl;
                     return true;
                     break;
 
                 case comment:
 
-                    // Only the tune-global comment field was asked for.
+                    // Only the file-global comment field was asked for.
 
                     if (firstTuneNo != start) {
-                        CERR_STIL_DEBUG << "getField() copying tune-global comment to resultbuf" << endl;
+                        CERR_STIL_DEBUG << "getField() copying file-global comment to resultbuf" << endl;
                         return getOneField(result, start, firstTuneNo, comment);
                     }
                     else {
-                        CERR_STIL_DEBUG << "getField() no tune-global comment" << endl;
+                        CERR_STIL_DEBUG << "getField() no file-global comment" << endl;
                         return false;
                     }
                     break;
@@ -1025,7 +1141,8 @@ STIL::getField(char *result, char *buffer, int tuneNo, STILField field)
 
         // Search for the requested tune number.
 
-        sprintf(tuneNoStr, "(#%d)", tuneNo);
+        snprintf(tuneNoStr, 7, "(#%d)", tuneNo);
+        tuneNoStr[7] = '\0';
         myTuneNo = strstr(start, tuneNoStr);
 
         if (myTuneNo != NULL) {
@@ -1036,14 +1153,19 @@ STIL::getField(char *result, char *buffer, int tuneNo, STILField field)
 
             // Where is the next one?
 
-            nextTuneNo = strstr(myTuneNo, "(#");
+            nextTuneNo = strstr(myTuneNo, "\n(#");
             if (nextTuneNo == NULL) {
                 // There is no next one - set pointer to end of entry.
                 nextTuneNo = start+strlen(start);
             }
+            else {
+                // The search included the \n - go beyond it.
+                nextTuneNo++;
+            }
 
             // Put the desired fields into the result (which may be 'all').
 
+            CERR_STIL_DEBUG << "getField() myTuneNo=" << myTuneNo << ", nextTuneNo=" << nextTuneNo << endl;
             return getOneField(result+strlen(result), myTuneNo, nextTuneNo, field);
         }
 
@@ -1063,9 +1185,12 @@ STIL::getOneField(char *result, char *start, char *end, STILField field)
 
     if ((end < start) || (*(end-1) != '\n')) {
         *result = '\0';
+        CERR_STIL_DEBUG << "getOneField() illegal parameters" << endl;
         return false;
     }
 
+    CERR_STIL_DEBUG << "getOneField() called, start=" << start << ", rest=" << field << endl;
+
     switch (field) {
 
         case all:
@@ -1079,6 +1204,11 @@ STIL::getOneField(char *result, char *start, char *end, STILField field)
             temp = strstr(start, _NAME_STR);
             break;
 
+        case author:
+
+            temp = strstr(start, _AUTHOR_STR);
+            break;
+
         case title:
 
             temp = strstr(start, _TITLE_STR);
@@ -1110,9 +1240,10 @@ STIL::getOneField(char *result, char *start, char *end, STILField field)
     // Search for the end of this field. This is done by finding
     // where the next field starts.
 
-    char *nextName, *nextTitle, *nextArtist, *nextComment, *nextField;
+    char *nextName, *nextAuthor, *nextTitle, *nextArtist, *nextComment, *nextField;
 
     nextName = strstr(temp+1, _NAME_STR);
+    nextAuthor = strstr(temp+1, _AUTHOR_STR);
     nextTitle = strstr(temp+1, _TITLE_STR);
     nextArtist = strstr(temp+1, _ARTIST_STR);
     nextComment = strstr(temp+1, _COMMENT_STR);
@@ -1123,6 +1254,10 @@ STIL::getOneField(char *result, char *start, char *end, STILField field)
         nextName = NULL;
     }
 
+    if ((nextAuthor != NULL) && (nextAuthor >= end)) {
+        nextAuthor = NULL;
+    }
+
     if ((nextTitle != NULL) && (nextTitle >= end)) {
         nextTitle = NULL;
     }
@@ -1140,6 +1275,13 @@ STIL::getOneField(char *result, char *start, char *end, STILField field)
 
     nextField = nextName;
 
+    if (nextField == NULL) {
+        nextField = nextAuthor;
+    }
+    else if ((nextAuthor != NULL) && (nextAuthor < nextField)) {
+        nextField = nextAuthor;
+    }
+
     if (nextField == NULL) {
         nextField = nextTitle;
     }
@@ -1191,15 +1333,6 @@ STIL::getStilLine(ifstream& infile, char *line)
     infile.getline(line, STIL_MAX_LINE_SIZE, STIL_EOL);
 }
 
-void
-STIL::addNewline(char *str)
-{
-    char *tempptr;
-
-    tempptr = str+strlen(str);
-    strcpy(tempptr, "\n");
-}
-
 void
 STIL::deleteDirList(dirList *dirs)
 {
