@@ -35,6 +35,9 @@ using namespace std;
 #ifdef XSID_HAVE_ARTSC
 #include "AudioDriverARTSC.h"
 #endif
+#ifdef XSID_HAVE_SDL
+#include "AudioDriverSDL.h"
+#endif
 #ifdef XSID_HAVE_ESD
 #include "AudioDriverESD.h"
 #endif
@@ -76,6 +79,9 @@ int main(int argc, char** argv)
 #ifdef XSID_HAVE_ARTSC
     AudioDrivers::addDriver( new AudioDriverARTSC );
 #endif
+#ifdef XSID_HAVE_SDL
+    AudioDrivers::addDriver( new AudioDriverSDL );
+#endif
         
     player = new Player;
     
@@ -285,6 +291,7 @@ int main(int argc, char** argv)
     delete config;
     delete player;
     
+    AudioDrivers::dispose();
     return qRet;
 }
 
